@@ -21,14 +21,20 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.ConnectException;
+import java.net.CookieHandler;
 import java.net.HttpURLConnection;
 import java.net.Socket;
 import java.net.SocketTimeoutException;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.net.UnknownHostException;
 import java.text.ParseException;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 import java.util.zip.GZIPInputStream;
 
@@ -41,6 +47,7 @@ import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 
+import org.tmatesoft.svn.core.SVNCancelException;
 import org.tmatesoft.svn.core.SVNErrorCode;
 import org.tmatesoft.svn.core.SVNErrorMessage;
 import org.tmatesoft.svn.core.SVNException;
@@ -62,6 +69,7 @@ import org.tmatesoft.svn.core.internal.wc.SVNCancellableOutputStream;
 import org.tmatesoft.svn.core.internal.wc.SVNErrorManager;
 import org.tmatesoft.svn.core.internal.wc.SVNFileUtil;
 import org.tmatesoft.svn.core.io.SVNRepository;
+import org.tmatesoft.svn.util.ISVNDebugLog;
 import org.tmatesoft.svn.util.SVNDebugLog;
 import org.tmatesoft.svn.util.SVNLogType;
 import org.xml.sax.EntityResolver;
@@ -76,7 +84,7 @@ import org.xml.sax.helpers.DefaultHandler;
  * @version 1.3
  * @author  TMate Software Ltd.
  */
-class HTTPConnection implements IHTTPConnection {
+public class HTTPConnection implements IHTTPConnection {
     
     private static final DefaultHandler DEFAULT_SAX_HANDLER = new DefaultHandler();
     private static EntityResolver NO_ENTITY_RESOLVER = new EntityResolver() {
@@ -104,6 +112,8 @@ class HTTPConnection implements IHTTPConnection {
     }
 
     private static SAXParserFactory ourSAXParserFactory;
+    private final static Map<String,List<String>> emptyHeader = Collections.unmodifiableMap(Collections.<String, List<String>>emptyMap());
+    
     private byte[] myBuffer;
     private SAXParser mySAXParser;
     private SVNURL myHost;
@@ -123,7 +133,6 @@ class HTTPConnection implements IHTTPConnection {
     private boolean myIsSpoolAll;
     private File mySpoolDirectory;
     private long myNextRequestTimeout;
-    private Collection<String> myCookies;
     private int myRequestCount;
     private HTTPStatus myLastStatus;
 
@@ -160,31 +169,108 @@ class HTTPConnection implements IHTTPConnection {
                 readTimeout = DEFAULT_HTTP_TIMEOUT;
             }
             if (proxyManager != null && proxyManager.getProxyHost() != null) {
-                myRepository.getDebugLog().logFine(SVNLogType.NETWORK, "Using proxy " + proxyManager.getProxyHost() + " (secured=" + myIsSecured + ")");
+                final ISVNDebugLog debugLog = myRepository.getDebugLog();
+                debugLog.logFine(SVNLogType.NETWORK, "Using proxy " + proxyManager.getProxyHost() + " (secured=" + myIsSecured + ")");
                 mySocket = SVNSocketFactory.createPlainSocket(proxyManager.getProxyHost(), proxyManager.getProxyPort(), connectTimeout, readTimeout, myRepository.getCanceller());
-                if (myProxyAuthentication == null) {
-                    myProxyAuthentication = new HTTPBasicAuthentication(proxyManager.getProxyUserName(), proxyManager.getProxyPassword(), myCharset);
-                }
                 myIsProxied = true;
                 if (myIsSecured) {
-                    HTTPRequest connectRequest = new HTTPRequest(myCharset);
-                    connectRequest.setConnection(this);
-                    connectRequest.initCredentials(myProxyAuthentication, "CONNECT", host + ":" + port);
-                    connectRequest.setProxyAuthentication(myProxyAuthentication.authenticate());
-                    connectRequest.setForceProxyAuth(true);
-                    connectRequest.dispatch("CONNECT", host + ":" + port, null, 0, 0, null);
-                    HTTPStatus status = connectRequest.getStatus();
-                    if (status.getCode() == HttpURLConnection.HTTP_OK) {
-                        myInputStream = null;
-                        myOutputStream = null;
-                        mySocket = SVNSocketFactory.createSSLSocket(keyManager != null ? new KeyManager[] { keyManager } : new KeyManager[0], trustManager, host, port, mySocket, readTimeout);
-                        proxyManager.acknowledgeProxyContext(true, null);
-                        return;
+                    int authAttempts = 0;
+                    boolean credentialsUsed = false;
+                    while(true) {
+                        if (mySocket == null) {
+                            mySocket = SVNSocketFactory.createPlainSocket(proxyManager.getProxyHost(), proxyManager.getProxyPort(), connectTimeout, readTimeout, myRepository.getCanceller());
+                            debugLog.logFine(SVNLogType.NETWORK, "proxy connection reopened");
+                        }
+                        HTTPRequest connectRequest = new HTTPRequest(myCharset);
+                        connectRequest.setConnection(this);
+                        if (myProxyAuthentication != null) {
+                            final String authToken = myProxyAuthentication.authenticate();
+                            connectRequest.setProxyAuthentication(authToken);
+                            debugLog.logFine(SVNLogType.NETWORK, "auth token set: " + authToken);
+                        }
+                        connectRequest.setForceProxyAuth(true);
+                        connectRequest.dispatch("CONNECT", host + ":" + port, null, 0, 0, null);
+                        HTTPStatus status = connectRequest.getStatus();
+                        
+                        if (status.getCode() == HttpURLConnection.HTTP_OK) {
+                            myInputStream = null;
+                            myOutputStream = null;
+                            myProxyAuthentication = null;
+                            mySocket = SVNSocketFactory.createSSLSocket(keyManager != null ? new KeyManager[] { keyManager } : new KeyManager[0], trustManager, host, port, mySocket, readTimeout);
+                            proxyManager.acknowledgeProxyContext(true, null);
+                            return;
+                        } else if (status.getCode() == HttpURLConnection.HTTP_PROXY_AUTH) {
+                            if (hasToCloseConnection(connectRequest.getResponseHeader())) {
+                                close();
+                                debugLog.logFine(SVNLogType.NETWORK, "Connection closed as requested by the response header");
+                            }
+                            authAttempts++;
+                            debugLog.logFine(SVNLogType.NETWORK, "authentication attempt #" + authAttempts);
+                            Collection<String> proxyAuthHeaders = connectRequest.getResponseHeader().getHeaderValues(HTTPHeader.PROXY_AUTHENTICATE_HEADER);
+                            Collection<String> authTypes = Arrays.asList("Basic", "Digest", "Negotiate", "NTLM");
+                            
+                            debugLog.logFine(SVNLogType.NETWORK, "authentication methods supported: " + authTypes);
+                            try {
+                                myProxyAuthentication = HTTPAuthentication.parseAuthParameters(proxyAuthHeaders, myProxyAuthentication, myCharset, authTypes, null, myRequestCount); 
+                            } catch (SVNException svne) {
+                                myRepository.getDebugLog().logFine(SVNLogType.NETWORK, svne);
+                                close();
+                                throw svne;
+                            }
+                            debugLog.logFine(SVNLogType.NETWORK, "authentication type chosen: " + myProxyAuthentication.getClass().getSimpleName());
+                            connectRequest.initCredentials(myProxyAuthentication, "CONNECT", host + ":" + port);
+                            
+                            HTTPNTLMAuthentication ntlmProxyAuth = null;
+                            HTTPNegotiateAuthentication negotiateProxyAuth = null;
+                            if (myProxyAuthentication instanceof HTTPNTLMAuthentication) {
+                                ntlmProxyAuth = (HTTPNTLMAuthentication) myProxyAuthentication;
+                                if (ntlmProxyAuth.isInType3State()) {
+                                    debugLog.logFine(SVNLogType.NETWORK, "continuation of NTLM authentication");
+                                    continue;
+                                }
+                            } else if (myProxyAuthentication instanceof HTTPNegotiateAuthentication) {
+                                negotiateProxyAuth = (HTTPNegotiateAuthentication) myProxyAuthentication;
+                                if (negotiateProxyAuth.isStarted()) {
+                                    debugLog.logFine(SVNLogType.NETWORK, "continuation of Negotiate authentication");
+                                    continue;
+                                }
+                            }
+                            
+                            if (ntlmProxyAuth != null && authAttempts == 1) {
+                                if (!ntlmProxyAuth.allowPropmtForCredentials()) {
+                                    continue;
+                                }
+                            }
+                            if (negotiateProxyAuth != null && !negotiateProxyAuth.needsLogin()) {
+                                debugLog.logFine(SVNLogType.NETWORK, "Negotiate will use existing credentials");
+                                continue;
+                            }
+
+                            if (!credentialsUsed) {
+                                myProxyAuthentication.setCredentials(new SVNPasswordAuthentication(proxyManager.getProxyUserName(), 
+                                        proxyManager.getProxyPassword(), false, myRepository.getLocation(), false));
+                                debugLog.logFine(SVNLogType.NETWORK, "explicit credentials set");
+                                credentialsUsed = true;
+                            } else {
+                                debugLog.logFine(SVNLogType.NETWORK, "no more credentials to try");
+                                SVNErrorMessage err = SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "HTTP proxy authorization failed");
+                                if (proxyManager != null) {
+                                    proxyManager.acknowledgeProxyContext(false, err);
+                                }
+                                SVNErrorManager.error(err, SVNLogType.NETWORK);
+                            }
+                        } else {                        
+                            SVNURL proxyURL = SVNURL.parseURIEncoded("http://" + proxyManager.getProxyHost() + ":" + proxyManager.getProxyPort());
+                            SVNErrorMessage err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "{0} request failed on ''{1}''", new Object[] {"CONNECT", proxyURL});
+                            proxyManager.acknowledgeProxyContext(false, err);
+                            SVNErrorManager.error(err, connectRequest.getErrorMessage(), SVNLogType.NETWORK);
+                        }
                     }
-                    SVNURL proxyURL = SVNURL.parseURIEncoded("http://" + proxyManager.getProxyHost() + ":" + proxyManager.getProxyPort());
-                    SVNErrorMessage err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "{0} request failed on ''{1}''", new Object[] {"CONNECT", proxyURL});
-                    proxyManager.acknowledgeProxyContext(false, err);
-                    SVNErrorManager.error(err, connectRequest.getErrorMessage(), SVNLogType.NETWORK);
+                } else if (proxyManager.getProxyUserName() != null && proxyManager.getProxyPassword() != null ){
+                    myProxyAuthentication = new HTTPBasicAuthentication("UTF-8");
+                    myProxyAuthentication.setCredentials(new SVNPasswordAuthentication(proxyManager.getProxyUserName(), 
+                            proxyManager.getProxyPassword(), false, myRepository.getLocation(), false));
+                    debugLog.logFine(SVNLogType.NETWORK, "explicit credentials set");
                 }
             } else {
                 myIsProxied = false;
@@ -261,7 +347,6 @@ class HTTPConnection implements IHTTPConnection {
     }
     
     public void clearAuthenticationCache() {
-        myCookies = null;
         myLastValidAuth = null;
         myTrustManager = null;
         myKeyManager = null;
@@ -304,7 +389,7 @@ class HTTPConnection implements IHTTPConnection {
         TrustManager trustManager = myTrustManager == null && authManager != null ? authManager.getTrustManager(myRepository.getLocation()) : myTrustManager;
         ISVNProxyManager proxyManager = authManager != null ? authManager.getProxyManager(myRepository.getLocation()) : null;
 
-        String sslRealm = "<" + myHost.getProtocol() + "://" + myHost.getHost() + ":" + myHost.getPort() + ">";
+        String sslRealm = composeRealm("");
         SVNAuthentication httpAuth = myLastValidAuth;
         boolean isAuthForced = authManager != null ? authManager.isAuthenticationForced() : false;
         if (httpAuth == null && isAuthForced) {
@@ -339,6 +424,12 @@ class HTTPConnection implements IHTTPConnection {
                 err = null;
                 String httpAuthResponse = null;
                 String proxyAuthResponse = null;
+                URI actualURI = null;
+                try {
+                    actualURI = new URI(myRepository.getLocation().toString());
+                } catch (URISyntaxException e) {
+                    myRepository.getDebugLog().logError(SVNLogType.NETWORK, e);
+                }
                 while(retryCount >= 0) {
                     connect(keyManager, trustManager, proxyManager);
                     request.reset();
@@ -361,13 +452,18 @@ class HTTPConnection implements IHTTPConnection {
                         request.setAuthentication(httpAuthResponse);
                     }
 
-                    if (myCookies != null && !myCookies.isEmpty()) {
-                        request.setCookies(myCookies);
+                    final CookieHandler cookieHandler = CookieHandler.getDefault();
+                    if (cookieHandler != null && actualURI != null) {
+                        final Map<String, List<String>> cookieHeader = cookieHandler.get(actualURI, emptyHeader);
+                        if (cookieHeader != null) {
+                            request.setCookies(cookieHeader);
+                        }
                     }
                     try {                        
                         request.dispatch(method, path, header, ok1, ok2, context);
                         break;
                     } catch (EOFException pe) {
+                        myRepository.getDebugLog().logFine(SVNLogType.NETWORK, pe);
                         // retry, EOF always means closed connection.
                         if (retryCount > 0) {
                             close();
@@ -378,21 +474,27 @@ class HTTPConnection implements IHTTPConnection {
                         retryCount--;
                     }
                 }
-                if (request.getResponseHeader().hasHeader(HTTPHeader.SET_COOKIE)) {
-                    myCookies = request.getResponseHeader().getHeaderValues(HTTPHeader.COOKIE);
+                final CookieHandler cookieHandler = CookieHandler.getDefault();
+                if(cookieHandler != null && actualURI != null){
+                    cookieHandler.put(actualURI, request.getResponseHeader().getRawHeaders());
                 }
                 myNextRequestTimeout = request.getNextRequestTimeout();
                 myLastStatus = request.getStatus();
             } catch (SSLHandshakeException ssl) {
                 myRepository.getDebugLog().logFine(SVNLogType.NETWORK, ssl);
                 close();
-	            if (ssl.getCause() instanceof SVNSSLUtil.CertificateNotTrustedException) {
+	            if (ssl.getCause() instanceof SVNSSLUtil.CertificateNotTrustedException 
+	                    || ssl.getCause() instanceof SVNSSLUtil.CertificateDoesNotConformConstraints) {
 		            SVNErrorManager.cancel(ssl.getCause().getMessage(), SVNLogType.NETWORK);
 	            }
                 SVNErrorMessage sslErr = SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "SSL handshake failed: ''{0}''", new Object[] { ssl.getMessage() }, SVNErrorMessage.TYPE_ERROR, ssl);
-		            if (keyManager != null) {
-			            keyManager.acknowledgeAndClearAuthentication(sslErr);
-		            }
+	            if (keyManager != null && keyManager.isInitialized()) {
+            		keyManager.acknowledgeAndClearAuthentication(sslErr);
+	            } else {
+            		sslErr = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, "SSL handshake failed: ''{0}''", new Object[] { ssl.getMessage() }, SVNErrorMessage.TYPE_ERROR, ssl);
+		            SVNErrorManager.error(sslErr, SVNLogType.NETWORK);
+	            	
+	            }
                 err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, ssl);
 	            continue;
             } catch (IOException e) {
@@ -410,11 +512,9 @@ class HTTPConnection implements IHTTPConnection {
                 } else if (e instanceof SVNCancellableOutputStream.IOCancelException) {
                     SVNErrorManager.cancel(e.getMessage(), SVNLogType.NETWORK);
                 } else if (e instanceof SSLException) {                   
-                    err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, 
-                            e);
+                    err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, e);
                 } else {
-                    err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, 
-                            e);
+                    err = SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, e);
                 }
             } catch (SVNException e) {
                 myRepository.getDebugLog().logFine(SVNLogType.NETWORK, e);
@@ -469,11 +569,11 @@ class HTTPConnection implements IHTTPConnection {
 
                 if (myProxyAuthentication instanceof HTTPNTLMAuthentication) {
                     ntlmProxyAuthIsRequired = true;
-                    HTTPNTLMAuthentication ntlmProxyAuth = (HTTPNTLMAuthentication)myProxyAuthentication;
+                    HTTPNTLMAuthentication ntlmProxyAuth = (HTTPNTLMAuthentication) myProxyAuthentication;
                     if (ntlmProxyAuth.isInType3State()) {
                         continue;
                     }
-                }
+                } 
 
                 err = SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, "HTTP proxy authorization failed");
                 if (proxyManager != null) {
@@ -488,7 +588,7 @@ class HTTPConnection implements IHTTPConnection {
                 Collection<String> authHeaderValues = request.getResponseHeader().getHeaderValues(HTTPHeader.AUTHENTICATE_HEADER);
                 if (authHeaderValues == null || authHeaderValues.size() == 0) {
                     err = request.getErrorMessage();
-                    myLastStatus.setError(SVNErrorMessage.create(SVNErrorCode.RA_DAV_REQUEST_FAILED, err.getMessageTemplate(), err.getRelatedObjects()));
+                    myLastStatus.setError(SVNErrorMessage.create(SVNErrorCode.RA_NOT_AUTHORIZED, err.getMessageTemplate(), err.getRelatedObjects()));
                     if ("LOCK".equalsIgnoreCase(method)) {
                         myLastStatus.getError().setChildErrorMessage(SVNErrorMessage.create(SVNErrorCode.UNSUPPORTED_FEATURE,
                                 "Probably you are trying to lock file in repository that only allows anonymous access"));
@@ -554,14 +654,16 @@ class HTTPConnection implements IHTTPConnection {
 
                 myLastValidAuth = null;
 
-                if (ntlmAuth != null && ntlmAuth.isNative() && authAttempts == 1) {
+                if (ntlmAuth != null && authAttempts == 1) {
                     /*
                      * if this is the first time we get HTTP_UNAUTHORIZED, NTLM is the target auth scheme
                      * and JNA is available, we should try a native auth mechanism first without calling 
                      * auth providers. 
-                     */
-                    continue;
-                }
+                     */                
+                    if (!ntlmAuth.allowPropmtForCredentials()) {
+                        continue;
+                    }
+               }
 
                 if (negoAuth != null && !negoAuth.needsLogin()) {
                     continue;
@@ -574,7 +676,7 @@ class HTTPConnection implements IHTTPConnection {
 
                 realm = myChallengeCredentials.getChallengeParameter("realm");
                 realm = realm == null ? "" : " " + realm;
-                realm = "<" + myHost.getProtocol() + "://" + myHost.getHost() + ":" + myHost.getPort() + ">" + realm;
+                realm = composeRealm(realm); 
                 
                 if (httpAuth == null) {
                     httpAuth = authManager.getFirstAuthentication(ISVNAuthenticationManager.PASSWORD, realm, myRepository.getLocation());
@@ -584,7 +686,7 @@ class HTTPConnection implements IHTTPConnection {
                 }
                 
                 if (httpAuth == null) {
-                    err = SVNErrorMessage.create(SVNErrorCode.CANCELLED, "HTTP authorization cancelled");
+                    err = SVNErrorMessage.create(SVNErrorCode.CANCELLED, new SVNCancelException(SVNErrorMessage.create(SVNErrorCode.CANCELLED, "ISVNAuthentication provider did not provide credentials; HTTP authorization cancelled.")));
                     break;
                 } 
                 if (httpAuth != null) {
@@ -642,8 +744,8 @@ class HTTPConnection implements IHTTPConnection {
                 myLastValidAuth = httpAuth;
             }
 
-            if (authManager instanceof ISVNAuthenticationManagerExt) {
-                ((ISVNAuthenticationManagerExt)authManager).acknowledgeConnectionSuccessful(myRepository.getLocation());
+            if (myLastStatus.getCodeClass() == 2 && authManager instanceof ISVNAuthenticationManagerExt) {
+                ((ISVNAuthenticationManagerExt) authManager).acknowledgeConnectionSuccessful(myRepository.getLocation(), method);
             }
 
             myLastStatus.setHeader(request.getResponseHeader());
@@ -663,6 +765,25 @@ class HTTPConnection implements IHTTPConnection {
         return null;
     }
 
+    private String composeRealm(String realm) {
+        final StringBuffer buffer = new StringBuffer();
+        buffer.append("<");
+        buffer.append(myHost.getProtocol());
+        buffer.append("://");
+        if (myHost.getUserInfo() != null && !"".equals(myHost.getUserInfo().trim())) {
+            buffer.append(myHost.getUserInfo());
+            buffer.append("@");
+        }
+        buffer.append(myHost.getHost());
+        buffer.append(":");
+        buffer.append(myHost.getPort());
+        buffer.append(">");
+        if (realm != null) {
+            buffer.append(realm);
+        }
+        return buffer.toString();
+    }
+
     private boolean isClearCredentialsOnClose(HTTPAuthentication auth) {
         return !(auth instanceof HTTPBasicAuthentication || auth instanceof HTTPDigestAuthentication 
                 || auth instanceof HTTPNegotiateAuthentication);
