@@ -11,28 +11,7 @@
  */
 package org.tmatesoft.svn.core.internal.io.fs;
 
-import java.io.ByteArrayOutputStream;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
-import java.nio.ByteBuffer;
-import java.nio.channels.FileChannel;
-import java.nio.charset.CharacterCodingException;
-import java.nio.charset.Charset;
-import java.nio.charset.CharsetDecoder;
-import java.nio.charset.CodingErrorAction;
-import java.nio.charset.MalformedInputException;
-import java.security.MessageDigest;
-import java.security.NoSuchAlgorithmException;
-import java.util.Map;
-import java.util.logging.Level;
-
-import org.tmatesoft.svn.core.SVNDepth;
-import org.tmatesoft.svn.core.SVNErrorCode;
-import org.tmatesoft.svn.core.SVNErrorMessage;
-import org.tmatesoft.svn.core.SVNException;
-import org.tmatesoft.svn.core.SVNProperties;
-import org.tmatesoft.svn.core.SVNPropertyValue;
+import org.tmatesoft.svn.core.*;
 import org.tmatesoft.svn.core.internal.util.SVNHashMap;
 import org.tmatesoft.svn.core.internal.wc.SVNErrorManager;
 import org.tmatesoft.svn.core.internal.wc.SVNFileUtil;
@@ -40,6 +19,15 @@ import org.tmatesoft.svn.core.io.SVNRepository;
 import org.tmatesoft.svn.util.SVNDebugLog;
 import org.tmatesoft.svn.util.SVNLogType;
 
+import java.io.*;
+import java.nio.ByteBuffer;
+import java.nio.channels.FileChannel;
+import java.nio.charset.*;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
+import java.util.Map;
+import java.util.logging.Level;
+
 
 /**
  * @version 1.3
@@ -48,8 +36,11 @@ import org.tmatesoft.svn.util.SVNLogType;
 public class FSFile {
     
     private File myFile;
+    private byte[] myData;
+    private int myOffset;
+    private int myLength;
     private FileChannel myChannel;
-    private FileInputStream myInputStream;
+    private InputStream myInputStream;
     private long myPosition;
     
     private long myBufferPosition;
@@ -61,6 +52,24 @@ public class FSFile {
     
     public FSFile(File file) {
         myFile = file;
+        myData = null;
+        myPosition = 0;
+        myBufferPosition = 0;
+        myBuffer = ByteBuffer.allocate(1024);
+        myReadLineBuffer = ByteBuffer.allocate(1024);
+        myDecoder = Charset.forName("UTF-8").newDecoder();
+        myDecoder = myDecoder.onMalformedInput(CodingErrorAction.REPORT).onUnmappableCharacter(CodingErrorAction.REPORT);
+    }
+
+    public FSFile(byte[] data) {
+        this(data, 0, data.length);
+    }
+
+    public FSFile(byte[] data, int offset, int length) {
+        myFile = null;
+        myData = data;
+        myOffset = offset;
+        myLength = length;
         myPosition = 0;
         myBufferPosition = 0;
         myBuffer = ByteBuffer.allocate(1024);
@@ -78,7 +87,7 @@ public class FSFile {
     }
 
     public long size() {
-        return myFile.length();
+        return myData == null ? myFile.length() : myLength;
     }
     
     public void resetDigest() {
@@ -118,7 +127,8 @@ public class FSFile {
     }
 
     public String readLine(int limit) throws SVNException {
-        allocateReadBuffer(limit);
+        long currentLimit = limit < 0 ? 1024 : limit; //if limit < 0, read line buffer should have infinite size
+        allocateReadBuffer((int) currentLimit);
         try {
             while(myReadLineBuffer.hasRemaining()) {
                 int b = read();
@@ -129,6 +139,15 @@ public class FSFile {
                     break;
                 }
                 myReadLineBuffer.put((byte) (b & 0XFF));
+                if (limit < 0 && !myReadLineBuffer.hasRemaining()) {
+                    //make myReadLineBuffer twice as larger
+                    byte[] oldArray = myReadLineBuffer.array();
+                    int oldLimit = (int) currentLimit;
+
+                    currentLimit = currentLimit * 2;
+                    allocateReadBuffer((int) currentLimit);
+                    myReadLineBuffer.put(oldArray, 0, oldLimit);
+                }
             }
             myReadLineBuffer.flip();
             return myDecoder.decode(myReadLineBuffer).toString();
@@ -268,7 +287,7 @@ public class FSFile {
         Map map = new SVNHashMap();
         String line;
         while(true) {
-            line = readLine(1024);
+            line = readLine(-1);
             if ("".equals(line)) {
                 break;
             }
@@ -291,7 +310,7 @@ public class FSFile {
     }
     
     public int read() throws IOException {
-        if (myChannel == null || myPosition < myBufferPosition || myPosition >= myBufferPosition + myBuffer.limit()) {
+        if ((myChannel == null && myInputStream == null) || myPosition < myBufferPosition || myPosition >= myBufferPosition + myBuffer.limit()) {
             if (fill() <= 0) {
                 return -1;
             }
@@ -358,21 +377,29 @@ public class FSFile {
             try {
                 myChannel.close();
             } catch (IOException e) {}
-            SVNFileUtil.closeFile(myInputStream);
-            myChannel = null;
-            myInputStream = null;
-            myPosition = 0;
-            myDigest = null;
         }
-        
+        SVNFileUtil.closeFile(myInputStream);
+        myChannel = null;
+        myInputStream = null;
+        myPosition = 0;
+        myDigest = null;
     }
     
     private int fill() throws IOException {
-        if (myChannel == null || myPosition < myBufferPosition || myPosition >= myBufferPosition + myBuffer.limit()) {
+        if ((myChannel == null && myInputStream == null) || myPosition < myBufferPosition || (myData == null && myPosition >= myBufferPosition + myBuffer.limit())) {
             myBufferPosition = myPosition;
-            getChannel().position(myBufferPosition);
+            if (myData == null) {
+                getChannel().position(myBufferPosition);
+            } else {
+                myInputStream = new ByteArrayInputStream(myData, myOffset, myLength);
+            }
             myBuffer.clear();
-            int read = getChannel().read(myBuffer);
+            int read;
+            if (myData == null) {
+                read = getChannel().read(myBuffer);
+            } else {
+                read = myInputStream.read(myBuffer.array(), myBuffer.position(), myBuffer.limit());
+            }
             myBuffer.position(0);
             myBuffer.limit(read >= 0 ? read : 0);
             return read;
@@ -389,9 +416,13 @@ public class FSFile {
     }
     
     private FileChannel getChannel() throws IOException {
+        if (myData != null) {
+            return null;
+        }
         if (myChannel == null) {
-            myInputStream = SVNFileUtil.createFileInputStream(myFile);
-            myChannel = myInputStream.getChannel();
+            final FileInputStream fileInputStream = SVNFileUtil.createFileInputStream(myFile);
+            myChannel = fileInputStream.getChannel();
+            myInputStream = fileInputStream;
         }
         return myChannel;
     }
