@@ -44,75 +44,8 @@ import org.tmatesoft.svn.core.internal.wc17.db.SVNWCDbRoot;
 import org.tmatesoft.svn.core.internal.wc17.db.SvnWcDbPristines;
 import org.tmatesoft.svn.core.internal.wc2.ISvnCommitRunner;
 import org.tmatesoft.svn.core.internal.wc2.SvnWcGeneration;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryCatImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryCopyRevisionPropertiesImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryCreateImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryDumpImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryFilterImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetAuthorImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetChangedDirectoriesImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetChangedImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetDateImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetDiffImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetHistoryImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetInfoImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetLockImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetLogImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetPropertiesImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetPropertyImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetRevisionPropertiesImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetRevisionPropertyImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetTreeImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetUUIDImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryGetYoungestImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryHotCopyImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryInitializeImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryListLocksImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryListTransactionsImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryLoadImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryPackImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryRecoverImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryRemoveLocksImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryRemoveTransactionsImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositorySetUUIDImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositorySyncInfoImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositorySynchronizeImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryUpgradeImpl;
-import org.tmatesoft.svn.core.internal.wc2.admin.SvnRepositoryVerifyImpl;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgAdd;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgCanonicalizeUrls;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgCat;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgCheckout;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgCleanup;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgCommit;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgDiff;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgExport;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgGetChangelistPaths;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgGetInfo;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgGetMergeInfo;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgGetProperties;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgGetStatus;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgGetStatusSummary;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgLogMergeInfo;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgMarkReplaced;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgMerge;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgMergePegged;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgMergeReintegrate;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgRelocate;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgRemove;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgReposToWcCopy;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgResolve;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgRevert;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgSetChangelist;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgSetLock;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgSetProperty;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgSuggestMergeSources;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgSwitch;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgUnlock;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgUpdate;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgUpgrade;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgWcToReposCopy;
-import org.tmatesoft.svn.core.internal.wc2.ng.SvnNgWcToWcCopy;
+import org.tmatesoft.svn.core.internal.wc2.admin.*;
+import org.tmatesoft.svn.core.internal.wc2.ng.*;
 import org.tmatesoft.svn.core.internal.wc2.old.SvnOldAdd;
 import org.tmatesoft.svn.core.internal.wc2.old.SvnOldAnnotate;
 import org.tmatesoft.svn.core.internal.wc2.old.SvnOldCanonicalizeUrls;
@@ -150,7 +83,6 @@ import org.tmatesoft.svn.core.internal.wc2.remote.SvnNgReposToReposCopy;
 import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteAnnotate;
 import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteCat;
 import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteDiff;
-import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteDiffSummarize;
 import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteExport;
 import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteGetInfo;
 import org.tmatesoft.svn.core.internal.wc2.remote.SvnRemoteGetProperties;
@@ -169,41 +101,7 @@ import org.tmatesoft.svn.core.wc.ISVNOptions;
 import org.tmatesoft.svn.core.wc.ISVNRepositoryPool;
 import org.tmatesoft.svn.core.wc.SVNEvent;
 import org.tmatesoft.svn.core.wc.SVNWCUtil;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryCat;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryCopyRevisionProperties;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryCreate;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryDump;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryFilter;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetAuthor;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetChanged;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetChangedDirectories;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetDate;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetDiff;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetHistory;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetInfo;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetLock;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetLog;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetProperties;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetProperty;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetRevisionProperties;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetRevisionProperty;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetTree;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetUUID;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryGetYoungest;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryHotCopy;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryInitialize;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryListLocks;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryListTransactions;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryLoad;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryPack;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryRecover;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryRemoveLocks;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryRemoveTransactions;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositorySetUUID;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositorySyncInfo;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositorySynchronize;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryUpgrade;
-import org.tmatesoft.svn.core.wc2.admin.SvnRepositoryVerify;
+import org.tmatesoft.svn.core.wc2.admin.*;
 import org.tmatesoft.svn.util.SVNLogType;
 
 /**
@@ -332,7 +230,7 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
         registerOperationRunner(SvnCat.class, new SvnNgCat());
         registerOperationRunner(SvnCat.class, new SvnOldCat());
 
-        registerOperationRunner(SvnDiffSummarize.class, new SvnRemoteDiffSummarize());
+        registerOperationRunner(SvnDiffSummarize.class, new SvnNgDiffSummarize());
         
         registerOperationRunner(SvnCopy.class, new SvnNgWcToWcCopy());
         registerOperationRunner(SvnCopy.class, new SvnNgReposToWcCopy());
@@ -423,7 +321,8 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
         registerOperationRunner(SvnRepositoryGetDiff.class, new SvnRepositoryGetDiffImpl());
         registerOperationRunner(SvnRepositoryGetHistory.class, new SvnRepositoryGetHistoryImpl());
         registerOperationRunner(SvnRepositoryGetTree.class, new SvnRepositoryGetTreeImpl());
-        
+        registerOperationRunner(SvnRepositoryGetFileSize.class, new SvnRepositoryGetFileSizeImpl());
+
         registerOperationRunner(SvnUpgrade.class, new SvnOldUpgrade());
         registerOperationRunner(SvnUpgrade.class, new SvnNgUpgrade());
 
@@ -1204,6 +1103,14 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
         return new SvnRepositoryGetRevisionProperties(this);
     }
 
+    /**
+     * Creates operation for retrieving file size from the repository
+     * @return new <code>SvnRepositoryGetFileSize</code> object
+     */
+    public SvnRepositoryGetFileSize createGetRepositoryFileSize() {
+        return new SvnRepositoryGetFileSize(this);
+    }
+
     /**
      * Creates get status summary operation.
      * @return new <code>SvnStatusSummary</code> object
@@ -1212,6 +1119,10 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
         return new SvnGetStatusSummary(this);
     }
 
+    public SvnSetWCDbVersion createSetWCDbVersion() {
+        return new SvnSetWCDbVersion(this);
+    }
+
     /**
      * Sets whether to dispose repository pool on {@link #dispose()} call.
      * This flag has sense only if <code>repositoryPool</code> field is not <code>null</code>.
@@ -1229,7 +1140,10 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
             SVNWCContext wcContext = null;
             runLevel++;
             try {
-                wcContext = obtainWcContext();
+                wcContext = obtainWcContext(operation);
+                if (runLevel == 1 && wcContext != null) {
+                    wcContext.setSqliteJournalMode(operation.getSqliteJournalMode());
+                }
                 runner.setWcContext(wcContext);
                 getOperationHandler().beforeOperation(operation);
                 Object result = runner.run(operation);
@@ -1243,6 +1157,9 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
             } finally {
                 runLevel--;
                 if (runLevel == 0) {
+                    if (wcContext != null) {
+                        wcContext.setSqliteJournalMode(null);
+                    }
                     releaseWcContext(wcContext);
                 }
             }
@@ -1264,10 +1181,11 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
         }
     }
 
-    private SVNWCContext obtainWcContext() {
+    private SVNWCContext obtainWcContext(SvnOperation<?> operation) {
         if (wcContext == null) {
-            wcContext = new SVNWCContext(getOptions(), getEventHandler());
+            wcContext = new SVNWCContext(SVNWCDbOpenMode.ReadWrite, getOptions(), false, true, getEventHandler());
         }
+        wcContext.setOperation(operation);
         return wcContext;
     }
 
@@ -1633,7 +1551,7 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
             }
             SVNWCDb db = new SVNWCDb();
             try {
-                db.open(SVNWCDbOpenMode.ReadOnly, (ISVNOptions) null, true, false);
+                db.open(SVNWCDbOpenMode.ReadOnly, (ISVNOptions) null, false, false);
                 DirParsedInfo info = db.parseDir(path, Mode.ReadOnly, true, isAdditionMode);
                 if (info != null && SVNWCDbDir.isUsable(info.wcDbDir)) {
                     return SvnWcGeneration.V17;
@@ -1773,7 +1691,7 @@ public class SvnOperationFactory implements ISvnOperationOptionsProvider {
             SVNWCContext wcContext = null;
             runLevel++;
             try {
-                wcContext = obtainWcContext();
+                wcContext = obtainWcContext(operation);
                 runner.setWcContext(wcContext);
                 return ((ISvnCommitRunner) runner).collectCommitItems(operation);
             } finally {
