@@ -6,10 +6,13 @@ import org.junit.Ignore;
 import org.junit.Test;
 import org.tmatesoft.svn.core.*;
 import org.tmatesoft.svn.core.auth.BasicAuthenticationManager;
+import org.tmatesoft.svn.core.internal.wc.DefaultSVNOptions;
 import org.tmatesoft.svn.core.internal.wc.SVNExternal;
 import org.tmatesoft.svn.core.internal.wc.SVNFileListUtil;
 import org.tmatesoft.svn.core.internal.wc.SVNFileUtil;
 import org.tmatesoft.svn.core.internal.wc17.SVNWCContext;
+import org.tmatesoft.svn.core.internal.wc17.db.ISVNWCDb;
+import org.tmatesoft.svn.core.internal.wc17.db.statement.SVNWCDbSchema;
 import org.tmatesoft.svn.core.internal.wc2.compat.SvnCodec;
 import org.tmatesoft.svn.core.wc.*;
 import org.tmatesoft.svn.core.wc2.*;
@@ -102,7 +105,7 @@ public class MergeTest {
 
                 Assert.assertEquals("mine", TestUtil.readFileContentsString(file));
 
-                final Map<File,SvnStatus> statuses = TestUtil.getStatuses(svnOperationFactory, workingCopyDirectory);
+                final Map<File, SvnStatus> statuses = TestUtil.getStatuses(svnOperationFactory, workingCopyDirectory);
                 Assert.assertEquals(SVNStatusType.STATUS_DELETED, statuses.get(file).getNodeStatus());
                 Assert.assertFalse(statuses.get(file).isConflicted());
             }
@@ -135,9 +138,7 @@ public class MergeTest {
 
             workingCopy.delete(file);
 
-            final SvnUpdate update = svnOperationFactory.createUpdate();
-            update.setSingleTarget(SvnTarget.fromFile(workingCopy.getWorkingCopyDirectory()));
-            update.run();
+            update(svnOperationFactory, workingCopy);
 
             final SvnGetInfo getInfo = svnOperationFactory.createGetInfo();
             getInfo.setSingleTarget(SvnTarget.fromFile(file));
@@ -146,15 +147,21 @@ public class MergeTest {
             final Collection<SVNConflictDescription> conflicts = svnInfo.getWcInfo().getConflicts();
             Assert.assertEquals(1, conflicts.size());
 
-            final SVNTreeConflictDescription conflict = (SVNTreeConflictDescription)conflicts.iterator().next();
+            final SVNTreeConflictDescription conflict = (SVNTreeConflictDescription) conflicts.iterator().next();
             Assert.assertEquals(SVNConflictAction.DELETE, conflict.getConflictAction());
             Assert.assertEquals(SVNConflictReason.DELETED, conflict.getConflictReason());
             Assert.assertEquals(url, conflict.getSourceLeftVersion().getRepositoryRoot());
-            Assert.assertEquals(url, conflict.getSourceRightVersion().getRepositoryRoot());
+            Assert.assertNull(conflict.getSourceRightVersion());
 
             final SvnSchedule schedule = svnInfo.getWcInfo().getSchedule();
             Assert.assertEquals(SvnSchedule.NORMAL, schedule);
 
+            final Map<File, SvnStatus> statuses = TestUtil.getStatuses(svnOperationFactory, workingCopy.getWorkingCopyDirectory());
+            final SvnStatus fileStatus = statuses.get(file);
+
+            Assert.assertTrue(fileStatus.isConflicted());
+            Assert.assertEquals(SVNStatusType.STATUS_CONFLICTED, fileStatus.getNodeStatus());
+
         } finally {
             svnOperationFactory.dispose();
             sandbox.dispose();
@@ -761,7 +768,6 @@ public class MergeTest {
     }
 
     @Test
-    @Ignore
     public void testConflictOnFileExternalUpdate() throws Exception {
         final TestOptions options = TestOptions.getInstance();
 
@@ -824,6 +830,338 @@ public class MergeTest {
         }
     }
 
+    @Test
+    public void testMergeRecordsActualPropertiesForFileWithMergeInfo() throws Exception {
+        final TestOptions options = TestOptions.getInstance();
+
+        Assume.assumeTrue(TestUtil.isNewWorkingCopyTest());
+
+        final SvnOperationFactory svnOperationFactory = new SvnOperationFactory();
+        final Sandbox sandbox = Sandbox.createWithCleanup(getTestName() + ".testMergeRecordsActualPropertiesForFileWithMergeInfo", options);
+        try {
+            final SVNURL url = sandbox.createSvnRepository();
+
+            final CommitBuilder commitBuilder1 = new CommitBuilder(url);
+            commitBuilder1.addFile("directory/file");
+            commitBuilder1.setFileProperty("directory/file", SVNProperty.MERGE_INFO, SVNPropertyValue.create("/somefile:1"));
+            commitBuilder1.commit();
+
+            final CommitBuilder commitBuilder2 = new CommitBuilder(url);
+            commitBuilder2.addFileByCopying("directory/copiedFile", "directory/file");
+            commitBuilder2.delete("directory/file");
+            commitBuilder2.commit();
+
+            final CommitBuilder commitBuilder3 = new CommitBuilder(url);
+            commitBuilder3.addDirectoryByCopying("copiedDirectory", "directory", 1);
+            commitBuilder3.commit();
+
+            final WorkingCopy workingCopy = sandbox.checkoutNewWorkingCopy(url.appendPath("copiedDirectory", false));
+            final File workingCopyDirectory = workingCopy.getWorkingCopyDirectory();
+
+            final SvnMerge merge = svnOperationFactory.createMerge();
+            merge.addRevisionRange(SvnRevisionRange.create(SVNRevision.create(1), SVNRevision.create(2)));
+            merge.setSource(SvnTarget.fromURL(url.appendPath("directory", false)), false);
+            merge.setSingleTarget(SvnTarget.fromFile(workingCopyDirectory));
+            merge.run();
+
+            Assert.assertEquals(2, TestUtil.getTableSize(workingCopy, SVNWCDbSchema.ACTUAL_NODE.name()));
+
+        } finally {
+            svnOperationFactory.dispose();
+            sandbox.dispose();
+        }
+    }
+
+    @Test
+    public void testImplicitGapIsExcludedFromMerging() throws Exception {
+        //SVNKIT-305
+        final TestOptions options = TestOptions.getInstance();
+
+        final SvnOperationFactory svnOperationFactory = new SvnOperationFactory();
+        final Sandbox sandbox = Sandbox.createWithCleanup(getTestName() + ".testImplicitGapIsExcludedFromMerging", options);
+        try {
+            final SVNURL url = sandbox.createSvnRepository();
+
+            final CommitBuilder commitBuilder1 = new CommitBuilder(url);
+            commitBuilder1.addFile("trunk/file");
+            commitBuilder1.commit();
+
+            final CommitBuilder commitBuilder2 = new CommitBuilder(url);
+            commitBuilder2.delete("trunk/file");
+            commitBuilder2.commit();
+
+            final CommitBuilder commitBuilder3 = new CommitBuilder(url);
+            commitBuilder3.addDirectoryByCopying("branches/branch", "trunk", 1);
+            commitBuilder3.commit();
+
+            final CommitBuilder commitBuilder4 = new CommitBuilder(url); //implicit src gap
+            commitBuilder4.commit();
+
+            final CommitBuilder commitBuilder5 = new CommitBuilder(url);
+            commitBuilder5.delete("branches/branch");
+            commitBuilder5.commit();
+
+            final CommitBuilder commitBuilder6 = new CommitBuilder(url);
+            commitBuilder6.addDirectoryByCopying("branches/branch", "trunk", 3);
+            commitBuilder6.commit();
+
+            final CommitBuilder commitBuilder7 = new CommitBuilder(url);
+            commitBuilder7.addFileByCopying("trunk/file", "trunk/file", 1);
+            commitBuilder7.commit();
+
+            final WorkingCopy workingCopy = sandbox.checkoutNewWorkingCopy(url.appendPath("trunk", false));
+
+            final SVNClientManager clientManager = SVNClientManager.newInstance();
+            try {
+                final SVNDiffClient diffClient = clientManager.getDiffClient();
+
+                workingCopy.setProperty(workingCopy.getFile("file"), SVNProperty.MERGE_INFO, SVNPropertyValue.create("/branches/branch/file:3")); //the value doesn't matter
+                workingCopy.setProperty(workingCopy.getWorkingCopyDirectory(), SVNProperty.MERGE_INFO, SVNPropertyValue.create("/branches/branch:6-7"));
+                Assert.assertEquals(8, workingCopy.commit(""));
+                workingCopy.updateToRevision(-1);
+
+                diffClient.doMerge(url.appendPath("branches/branch", false), SVNRevision.HEAD, Arrays.asList(
+                        new SVNRevisionRange(SVNRevision.create(1), SVNRevision.HEAD)
+                ), workingCopy.getWorkingCopyDirectory(), SVNDepth.INFINITY, true, false, false, false);
+
+                Assert.assertEquals(9, workingCopy.commit(""));
+
+            } finally {
+                clientManager.dispose();
+            }
+
+        } finally {
+            svnOperationFactory.dispose();
+            sandbox.dispose();
+        }
+    }
+
+    @Test
+    public void testMergeAfterShallowMergeProducesCorrectMergeinfo() throws Exception {
+        final TestOptions options = TestOptions.getInstance();
+
+        final SvnOperationFactory svnOperationFactory = new SvnOperationFactory();
+        final Sandbox sandbox = Sandbox.createWithCleanup(getTestName() + ".testMergeAfterShallowMergeProducesCorrectMergeinfo", options);
+        try {
+            final SVNURL url = sandbox.createSvnRepository();
+
+            final CommitBuilder commitBuilder1 = new CommitBuilder(url);
+            commitBuilder1.addFile("directory/subdirectory/file");
+            commitBuilder1.commit();
+
+            final CommitBuilder commitBuilder2 = new CommitBuilder(url);
+            commitBuilder2.addDirectoryByCopying("copiedDirectory", "directory");
+            commitBuilder2.commit();
+
+            final CommitBuilder commitBuilder3 = new CommitBuilder(url);
+            commitBuilder3.changeFile("directory/subdirectory/file", "contents".getBytes());
+            commitBuilder3.commit();
+
+            final WorkingCopy workingCopy = sandbox.checkoutNewWorkingCopy(url);
+
+            final SvnMerge shallowMerge = svnOperationFactory.createMerge();
+            shallowMerge.addRevisionRange(SvnRevisionRange.create(SVNRevision.create(1), SVNRevision.HEAD));
+            shallowMerge.setSource(SvnTarget.fromURL(url.appendPath("directory", false)), false);
+            shallowMerge.setSingleTarget(SvnTarget.fromFile(workingCopy.getFile("copiedDirectory")));
+            shallowMerge.setDepth(SVNDepth.FILES);
+            shallowMerge.run();
+
+            final SvnMerge merge = svnOperationFactory.createMerge();
+            merge.addRevisionRange(SvnRevisionRange.create(SVNRevision.create(1), SVNRevision.HEAD));
+            merge.setSource(SvnTarget.fromURL(url.appendPath("directory", false)), false);
+            merge.setSingleTarget(SvnTarget.fromFile(workingCopy.getFile("copiedDirectory")));
+            merge.setDepth(SVNDepth.INFINITY);
+            merge.run();
+
+            final SvnGetProperties getProperties = svnOperationFactory.createGetProperties();
+            getProperties.setSingleTarget(SvnTarget.fromFile(workingCopy.getFile("copiedDirectory")));
+            final SVNProperties properties = getProperties.run();
+
+            Assert.assertEquals("/directory:2-3", SVNPropertyValue.getPropertyAsString(properties.getSVNPropertyValue(SVNProperty.MERGE_INFO)));
+
+        } finally {
+            svnOperationFactory.dispose();
+            sandbox.dispose();
+        }
+    }
+
+    @Test
+    public void testPostponingConflictDoesntCreateEditedFile() throws Exception {
+        final TestOptions options = TestOptions.getInstance();
+
+        final SvnOperationFactory svnOperationFactory = new SvnOperationFactory();
+        final Sandbox sandbox = Sandbox.createWithCleanup(getTestName() + ".testPostponingConflictDoesntCreateEditedFile", options);
+        try {
+            final SVNURL url = sandbox.createSvnRepository();
+
+            final CommitBuilder commitBuilder1 = new CommitBuilder(url);
+            commitBuilder1.addFile("file", "base".getBytes());
+            SVNCommitInfo commitInfo1 = commitBuilder1.commit();
+
+            final CommitBuilder commitBuilder2 = new CommitBuilder(url);
+            commitBuilder2.changeFile("file", "theirs".getBytes());
+            commitBuilder2.commit();
+
+            final WorkingCopy workingCopy = sandbox.checkoutNewWorkingCopy(url, commitInfo1.getNewRevision());
+            final File workingCopyDirectory = workingCopy.getWorkingCopyDirectory();
+
+            final File file = workingCopy.getFile("file");
+            TestUtil.writeFileContentsString(file, "mine");
+
+            final File anyFile = workingCopy.getFile("anyFile");
+            TestUtil.writeFileContentsString(anyFile, "anyFile");
+
+            final DefaultSVNOptions svnOptions = new DefaultSVNOptions();
+            svnOptions.setConflictHandler(new ISVNConflictHandler() {
+                public SVNConflictResult handleConflict(SVNConflictDescription conflictDescription) throws SVNException {
+                    return new SVNConflictResult(SVNConflictChoice.POSTPONE, anyFile);
+                }
+            });
+            svnOperationFactory.setOptions(svnOptions);
+
+            final SvnUpdate update = svnOperationFactory.createUpdate();
+            update.setSingleTarget(SvnTarget.fromFile(workingCopyDirectory));
+            update.run();
+
+            final File editedFile = workingCopy.getFile("file.edited");
+            Assert.assertFalse(editedFile.exists());
+
+            final SvnGetInfo getInfo = svnOperationFactory.createGetInfo();
+            getInfo.setSingleTarget(SvnTarget.fromFile(file));
+            final SvnInfo svnInfo = getInfo.run();
+
+            final Collection<SVNConflictDescription> conflicts = svnInfo.getWcInfo().getConflicts();
+            Assert.assertEquals(1, conflicts.size());
+
+        } finally {
+            svnOperationFactory.dispose();
+            sandbox.dispose();
+        }
+    }
+
+    @Test
+    public void testMergeReintegrate() throws Exception {
+        final TestOptions options = TestOptions.getInstance();
+
+        final SvnOperationFactory svnOperationFactory = new SvnOperationFactory();
+        final Sandbox sandbox = Sandbox.createWithCleanup(getTestName() + ".testMergeReintegrate", options);
+        try {
+            final SVNURL url = sandbox.createSvnRepository();
+
+            final CommitBuilder commitBuilder1 = new CommitBuilder(url);
+            commitBuilder1.addDirectory("trunk");
+            commitBuilder1.addDirectory("branches");
+            commitBuilder1.addDirectory("tags");
+            commitBuilder1.commit();
+
+            final CommitBuilder commitBuilder2 = new CommitBuilder(url);
+            commitBuilder2.addFile("trunk/file", "string1".getBytes());
+            commitBuilder2.commit();
+
+            final CommitBuilder commitBuilder3 = new CommitBuilder(url);
+            commitBuilder3.addDirectoryByCopying("branches/branch", "trunk");
+            commitBuilder3.commit();
+
+            final CommitBuilder commitBuilder4 = new CommitBuilder(url);
+            commitBuilder4.changeFile("trunk/file", "string2".getBytes());
+            commitBuilder4.commit();
+
+            final CommitBuilder commitBuilder5 = new CommitBuilder(url);
+            commitBuilder5.setDirectoryProperty("branches/branch", SVNProperty.MERGE_INFO, SVNPropertyValue.create("/trunk:4"));
+            commitBuilder5.changeFile("branches/branch/file", "string2".getBytes());
+            commitBuilder5.commit();
+
+            final CommitBuilder commitBuilder6 = new CommitBuilder(url);
+            commitBuilder6.changeFile("branches/branch/file", "string3".getBytes());
+            commitBuilder6.commit();
+
+            final SVNURL trunkUrl = url.appendPath("trunk", false);
+            final SVNURL branchUrl = url.appendPath("branches/branch", false);
+            final WorkingCopy workingCopy = sandbox.checkoutNewWorkingCopy(trunkUrl);
+            final File workingCopyDirectory = workingCopy.getWorkingCopyDirectory();
+            final File file = workingCopy.getFile("file");
+
+            final SVNURL trunkFileUrl = trunkUrl.appendPath("file", false);
+            final SVNURL branchFileUrl = branchUrl.appendPath("file", false);
+
+            final SvnMerge merge = svnOperationFactory.createMerge();
+            merge.addRevisionRange(SvnRevisionRange.create(SVNRevision.create(0), SVNRevision.HEAD));
+            merge.setSingleTarget(SvnTarget.fromFile(workingCopyDirectory));
+            merge.setSource(SvnTarget.fromURL(branchUrl), false);
+            merge.run();
+
+            final Map<File, SvnStatus> statuses = TestUtil.getStatuses(svnOperationFactory, workingCopyDirectory);
+            for (Map.Entry<File, SvnStatus> entry : statuses.entrySet()) {
+                final SvnStatus status = entry.getValue();
+                boolean conflicted = status.isConflicted();
+
+                Assert.assertFalse(conflicted);
+            }
+
+        } finally {
+            svnOperationFactory.dispose();
+            sandbox.dispose();
+        }
+    }
+
+    @Test
+    public void testMergeWC17() throws Exception {
+        //SVNKIT-430
+        final TestOptions options = TestOptions.getInstance();
+
+        final SvnOperationFactory svnOperationFactory = new SvnOperationFactory();
+        final Sandbox sandbox = Sandbox.createWithCleanup(getTestName() + ".testMergeWC17", options);
+        try {
+            final SVNURL url = sandbox.createSvnRepository();
+
+            final CommitBuilder commitBuilder1 = new CommitBuilder(url);
+            commitBuilder1.addFile("trunk/file");
+            commitBuilder1.commit();
+
+            final CommitBuilder commitBuilder2 = new CommitBuilder(url);
+            commitBuilder2.addDirectoryByCopying("branches/branch", "trunk");
+            commitBuilder2.commit();
+
+            final CommitBuilder commitBuilder3 = new CommitBuilder(url);
+            commitBuilder3.changeFile("trunk/file", "ours".getBytes());
+            commitBuilder3.changeFile("branches/branch/file", "theirs".getBytes());
+            commitBuilder3.commit();
+
+            final SVNURL trunkUrl = url.appendPath("trunk", false);
+            final SVNURL branchUrl = url.appendPath("branches/branch", false);
+            final File workingCopyDirectory = sandbox.createDirectory("wc");
+            final File file = new File(workingCopyDirectory, "file");
+
+            final SvnCheckout checkout = svnOperationFactory.createCheckout();
+            checkout.setSource(SvnTarget.fromURL(trunkUrl));
+            checkout.setSingleTarget(SvnTarget.fromFile(workingCopyDirectory));
+            checkout.setTargetWorkingCopyFormat(ISVNWCDb.WC_FORMAT_17);
+            checkout.run();
+
+            final SvnMerge merge = svnOperationFactory.createMerge();
+            merge.addRevisionRange(SvnRevisionRange.create(SVNRevision.create(0), SVNRevision.HEAD));
+            merge.setSingleTarget(SvnTarget.fromFile(workingCopyDirectory));
+            merge.setSource(SvnTarget.fromURL(branchUrl), false);
+            merge.run();
+
+            final Map<File, SvnStatus> statuses = TestUtil.getStatuses(svnOperationFactory, workingCopyDirectory);
+            final SvnStatus fileStatus = statuses.get(file);
+
+            Assert.assertTrue(fileStatus.isConflicted());
+            Assert.assertEquals(ISVNWCDb.WC_FORMAT_17, fileStatus.getWorkingCopyFormat());
+
+        } finally {
+            svnOperationFactory.dispose();
+            sandbox.dispose();
+        }
+    }
+
+    private void update(SvnOperationFactory svnOperationFactory, WorkingCopy workingCopy) throws SVNException {
+        final SvnUpdate update = svnOperationFactory.createUpdate();
+        update.setSingleTarget(SvnTarget.fromFile(workingCopy.getWorkingCopyDirectory()));
+        update.run();
+    }
+
     private void runResolve(SvnOperationFactory svnOperationFactory, File file, SVNConflictChoice resolution) throws SVNException {
         final SVNClientManager clientManager = SVNClientManager.newInstance(svnOperationFactory.getOptions(), svnOperationFactory.getRepositoryPool());
         try {
